@@ -7,11 +7,11 @@ from BlockDialog import BlockDialog
 from UploadDialog import UploadDialog
 from ProgressionBrowser import ProgressionBrowser
 from InstrumentBrowser import InstrumentBrowser
+from MovementScene import MovementScene
 from os import path, environ
 from mingus.core import diatonic
 import Options
 import Movements
-import math
 
 APP_NAME = "qtImproviser"
 APP_VERSION = "0.8"
@@ -27,193 +27,6 @@ class OptionClass:
 	verbose=True
 	meter = (4,4)
 
-class MovementScene(QtGui.QGraphicsScene):
-
-	IOFFSET = 50
-	BOXSIZE = 25
-
-	def __init__(self, main):
-		QtGui.QGraphicsScene.__init__(self)
-		self.main = main
-		self.ui = main.ui
-
-		# Brushes and pens
-		self.brush = QtGui.QBrush(QtCore.Qt.SolidPattern)
-		self.brush_dont_play = QtGui.QBrush(QtCore.Qt.white, QtCore.Qt.NoBrush)
-		self.brush_play = QtGui.QBrush(QtCore.Qt.red, QtCore.Qt.SolidPattern)
-		self.brush_percussion = QtGui.QBrush(QtCore.Qt.yellow, QtCore.Qt.SolidPattern)
-		self.brush_selection = QtGui.QBrush(QtGui.QColor(0, 0, 255).light(186), QtCore.Qt.SolidPattern)
-		self.box_pen = QtGui.QPen(self.brush, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
-		self.select_pen = QtGui.QPen(self.brush, 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
-
-	def mousePressEvent(self, ev):
-		self.pressed = ev.scenePos()
-
-	def mouseReleaseEvent(self, ev):
-		if self.pressed == ev.scenePos():
-			x, y = self.get_box_coords(ev)
-			if y >= 0:
-				self.ui.instruments.setCurrentRow(y)
-				self.update()
-			if x >= 0 and y >= 0:
-				print x, y
-
-	def get_box_coords(self, ev):
-		x, y = self.pressed.x(), self.pressed.y()
-		x = x - 200
-		y = y - self.IOFFSET
-		boxx = x / self.BOXSIZE
-		boxy = y / self.BOXSIZE
-		x = -1
-		y = -1
-		if boxy - math.floor(boxy) < 0.8:
-			y = int(math.floor(boxy))
-		if boxx - math.floor(boxx) < 0.8:
-			x = int(math.floor(boxx))
-		return (x, y)
-
-	def mouseDoubleClickEvent(self, ev):
-		if self.pressed == ev.scenePos():
-			x, y = self.get_box_coords(ev)
-			if x >= 0 and y >= 0:
-				self.ui.editinstrument.click()
-		
-		self.pressed = []
-
-	def plays(self, params, n):
-		start, end, step, global_end = -1, -1, -1, -1
-		if 'start' in params:
-			start = params['start']
-		if 'step' in params:
-			step = params['step']
-		if 'end' in params:
-			end = params['end']
-		if 'global_end' in params:
-			global_end = params['global_end']
-
-		if global_end != -1 and n >= global_end:
-			return False
-
-		if start == -1:
-			return True
-		if n < start:
-			return False
-
-		if end == -1 and step == -1:
-			return True
-
-		if step == -1:
-			if n >= start and n < end:
-				return True
-			return False
-
-		if end == -1:
-			n = (n - start) % (step * 2)
-			if n < step:
-				return True
-			return False
-
-		n = (n - start) % (end - start + step)
-		if n < end - start:
-			return True
-		return False
-
-	def get_progressions(self):
-		try:
-			prog = [Options.parse_progression(x) for x in self.main.get_progressions().split(",")]
-			prog_text = [ x.split()[0] for x in self.main.get_progressions().split(",")]
-			return (prog, prog_text)
-		except:
-			return ([], [])
-
-	def get_prog_block_index(self, prog):
-		duration = int(self.ui.duration.value())
-		prog_block_index, blocks, prog_index, offset = [], [], 0, 0
-		if self.ui.blocks.count() > 0 and len(prog) != 0:
-			for x in self.main.get_blocks().split(","):
-				parts = x.split()
-				params = Options.parse_block_params(parts[1:])
-				dur = duration
-				if 'duration' in params:
-					dur = params['duration']
-				for i in range(dur):
-					prog_block_index.append((offset, parts[0], prog_index))
-					prog_offset = 0
-					for itk, tk in enumerate(prog[prog_index]):
-						if itk != 0:
-							offset += (tk[0] - prog_offset) * len(prog[prog_index][itk - 1][1])
-						prog_offset = tk[0]
-				prog_index += 1
-				if prog_index >= len(prog):
-					prog_index =0
-		return (offset, prog_block_index)
-
-	def paint_prog_block_index(self):
-		prog, prog_text = self.get_progressions()
-		end, prog_block_index = self.get_prog_block_index(prog)
-		IOFFSET = self.IOFFSET
-		BOXSIZE = self.BOXSIZE
-
-		if len(prog) <= 0 or self.main.get_instruments() is None:
-			return end
-
-		for i, x in enumerate(prog_block_index):
-			offset, name, prog_index = x
-			blockend = end
-			if i != len(prog_block_index) - 1:
-				blockend = prog_block_index[i + 1][0]
-			if name != 'R':
-				self.addLine(QtCore.QLineF(200 + offset * BOXSIZE, IOFFSET - 10, 
-					195 + blockend * BOXSIZE, IOFFSET - 10), self.box_pen)
-				self.addLine(QtCore.QLineF(200 + offset * BOXSIZE, IOFFSET - 10, 
-					200 + offset * BOXSIZE, IOFFSET - 8), self.box_pen)
-				self.addLine(QtCore.QLineF(195 + blockend * BOXSIZE, IOFFSET - 10, 
-					195 + blockend * BOXSIZE, IOFFSET - 8), self.box_pen)
-				t = self.addText(name[:int((blockend-offset) * 2.5)], 
-					QtGui.QFont("", 12, 80))
-				t.translate(200 + BOXSIZE * offset, 3)
-				if blockend - offset > 2:
-					t = self.addText(prog_text[prog_index], QtGui.QFont("", 8, 40, True))
-					t.translate(200 + BOXSIZE * offset, IOFFSET - 25)
-		return end
-
-
-	def update(self):
-		IOFFSET = self.IOFFSET
-		BOXSIZE = self.BOXSIZE
-			
-		for x in self.items():
-			self.removeItem(x)
-		
-		end = self.paint_prog_block_index()
-	
-
-		sel = self.ui.instruments.currentRow()
-		if sel != -1:
-			r = self.addRect(QtCore.QRectF(-1, sel * BOXSIZE + IOFFSET - 1, self.width(), BOXSIZE - BOXSIZE / 5 + 1), self.select_pen, self.brush_selection)
-			r.setZValue(-50)
-
-		instr = self.main.get_instruments()
-		if instr is not None:
-			for i, x in enumerate(instr.split(",")):
-				parts = x.split()
-				name = parts[0]
-				params = Options.parse_instrument_params(parts[1:])
-				s = self.addText(name)
-				s.translate(0, i * BOXSIZE + IOFFSET)
-				for n in range(end + 1):
-					if self.plays(params, n):
-						b = self.brush_play
-						if 'channel' in params:
-							if params['channel'] == 9:
-								b = self.brush_percussion
-					else:
-						b = self.brush_dont_play
-					a = self.addRect(QtCore.QRectF(200 + n * BOXSIZE, i * BOXSIZE + IOFFSET, BOXSIZE - BOXSIZE / 5, BOXSIZE - BOXSIZE / 5 ), self.box_pen,b)
-
-		self.ui.graphicsView.setScene(self)
-
-
 class ImproviserMainWindow(QtGui.QMainWindow):
 
 	lastfile = ""
@@ -244,22 +57,70 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.ui.key.addItem(x)
 				
 
-	def wresize(self):
-		print "resize"
+	def resizeEvent(self, ev):
+		self.scene.update()
 
 	def connect_widgets(self):
+		connect_dict = {
+			"clicked()": [
+				(self.ui.startbutton, self.start_simulation),
+				(self.ui.stopbutton, self.stop_simulation),
+				(self.ui.addblockbutton, self.add_block),
+				(self.ui.addprogressionbutton, self.add_progression),
+				(self.ui.addinstrumentbutton, self.add_instrument),
+				(self.ui.upinstrument, lambda: self.swap_list_item(\
+						self.ui.instruments)),
+				(self.ui.downinstrument, lambda: self.swap_list_item(\
+						self.ui.instruments, False)),
+				(self.ui.removeblock, self.remove_block),
+				(self.ui.clearblocks, lambda: self.ui.blocks.clear() or \
+						self.block_changed()),
+				(self.ui.upblock, lambda: self.swap_list_item(self.ui.blocks)),
+				(self.ui.downblock, lambda: self.swap_list_item(\
+						self.ui.blocks, False)),
+				(self.ui.removeinstrument, self.remove_instrument),
+				(self.ui.clearinstruments, lambda: self.ui.instruments.clear() \
+						or self.instrument_changed()),
+				(self.ui.editblock, self.edit_block),
+				(self.ui.editinstrument, self.edit_instrument),
+				(self.ui.editprogression, self.edit_progression),
+				(self.ui.removeprogression, self.remove_progression),
+				(self.ui.copyblock, self.copy_block),
+				(self.ui.copyinstrument, self.copy_instrument),
+				(self.ui.copyprogression, self.copy_progression),
+				(self.ui.upprogression, lambda: self.swap_list_item(\
+						self.ui.progressions)),
+				(self.ui.downprogression, lambda: self.swap_list_item(\
+						self.ui.progressions, False)),
+				(self.ui.clearprogressions, lambda: \
+						self.ui.progressions.clear() or \
+						self.progression_changed()),
+
+				],
+			"activated()": [
+				(self.ui.importprogressions, self.import_progression),
+				(self.ui.importinstruments, self.import_instruments),
+				(self.ui.importblocks, self.import_blocks),
+				(self.ui.exportprogressions, self.export_progression),
+				(self.ui.exportinstruments, self.export_instruments),
+				(self.ui.exportblocks, self.export_blocks),
+				(self.ui.action_About, self.show_about),
+				(self.ui.action_Load, self.load_dialog),
+				(self.ui.action_Save, self.save_file),
+				(self.ui.actionS_ave_as, self.save_dialog),
+				(self.ui.actionNew, self.set_defaults),
+				]
+
+				}
+
+		for x in connect_dict:
+			for widget, callback in connect_dict[x]:
+				self.connect(widget, QtCore.SIGNAL(x), callback)
+
+
 		self.connect(self,
 			QtCore.SIGNAL("resizeEvent(event)"),
 			lambda x: self.wresize())
-		self.connect(self.ui.startbutton, 
-			QtCore.SIGNAL("clicked()"), 
-			self.start_simulation)
-		self.connect(self.ui.stopbutton, 
-			QtCore.SIGNAL("clicked()"), 
-			self.stop_simulation)
-		self.connect(self.ui.addblockbutton,
-			QtCore.SIGNAL("clicked()"),
-			self.add_block)
 		self.connect(self.ui.instruments,
 			QtCore.SIGNAL("currentItemChanged(QListWidgetItem*, QListWidgetItem*)"),
 			lambda x,y: self.instrument_changed())
@@ -269,72 +130,6 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.connect(self.ui.progressions,
 			QtCore.SIGNAL("currentItemChanged(QListWidgetItem*, QListWidgetItem*)"),
 			lambda x,y: self.progression_changed())
-		self.connect(self.ui.addprogressionbutton,
-			QtCore.SIGNAL("clicked()"),
-			self.add_progression)
-		self.connect(self.ui.addinstrumentbutton,
-			QtCore.SIGNAL("clicked()"),
-			self.add_instrument)
-		self.connect(self.ui.upinstrument,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.swap_list_item(self.ui.instruments))
-		self.connect(self.ui.downinstrument,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.swap_list_item(self.ui.instruments, False))
-		self.connect(self.ui.removeblock,
-			QtCore.SIGNAL("clicked()"),
-			self.remove_block)
-		self.connect(self.ui.clearblocks,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.ui.blocks.clear() or self.block_changed())
-		self.connect(self.ui.upblock,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.swap_list_item(self.ui.blocks))
-		self.connect(self.ui.downblock,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.swap_list_item(self.ui.blocks, False))
-		self.connect(self.ui.removeinstrument,
-			QtCore.SIGNAL("clicked()"),
-			self.remove_instrument)
-		self.connect(self.ui.clearinstruments,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.ui.instruments.clear() or self.instrument_changed())
-		self.connect(self.ui.editblock,
-			QtCore.SIGNAL("clicked()"),
-			self.edit_block)
-		self.connect(self.ui.editinstrument,
-			QtCore.SIGNAL("clicked()"),
-			self.edit_instrument)
-		self.connect(self.ui.editprogression,
-			QtCore.SIGNAL("clicked()"),
-			self.edit_progression)
-		self.connect(self.ui.removeprogression,
-			QtCore.SIGNAL("clicked()"),
-			self.remove_progression)
-		self.connect(self.ui.upprogression,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.swap_list_item(self.ui.progressions))
-		self.connect(self.ui.downprogression,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.swap_list_item(self.ui.progressions, False))
-		self.connect(self.ui.clearprogressions,
-			QtCore.SIGNAL("clicked()"),
-			lambda: self.ui.progressions.clear() or self.progression_changed())
-		self.connect(self.ui.action_About,
-			QtCore.SIGNAL("activated()"),
-			self.show_about)
-		self.connect(self.ui.action_Load,
-			QtCore.SIGNAL("activated()"),
-			self.load_dialog)
-		self.connect(self.ui.action_Save,
-			QtCore.SIGNAL("activated()"),
-			self.save_file)
-		self.connect(self.ui.actionS_ave_as,
-			QtCore.SIGNAL("activated()"),
-			self.save_dialog)
-		self.connect(self.ui.actionNew,
-			QtCore.SIGNAL("activated()"),
-			self.set_defaults)
 		self.connect(self.ui.bpmslide,
 			QtCore.SIGNAL("sliderMoved(int)"),
 			lambda x: self.ui.bpm.setValue(x * 10))
@@ -353,33 +148,6 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.connect(self.ui.wildness,
 			QtCore.SIGNAL("valueChanged(double)"),
 			lambda x: self.ui.wildslide.setValue(int(x * 10)))
-		self.connect(self.ui.copyblock,
-			QtCore.SIGNAL("clicked()"),
-			self.copy_block)
-		self.connect(self.ui.copyinstrument,
-			QtCore.SIGNAL("clicked()"),
-			self.copy_instrument)
-		self.connect(self.ui.copyprogression,
-			QtCore.SIGNAL("clicked()"),
-			self.copy_progression)
-		self.connect(self.ui.importprogressions,
-			QtCore.SIGNAL("activated()"),
-			self.import_progression)
-		self.connect(self.ui.importinstruments,
-			QtCore.SIGNAL("activated()"),
-			self.import_instruments)
-		self.connect(self.ui.importblocks,
-			QtCore.SIGNAL("activated()"),
-			self.import_blocks)
-		self.connect(self.ui.exportprogressions,
-			QtCore.SIGNAL("activated()"),
-			self.export_progression)
-		self.connect(self.ui.exportinstruments,
-			QtCore.SIGNAL("activated()"),
-			self.export_instruments)
-		self.connect(self.ui.exportblocks,
-			QtCore.SIGNAL("activated()"),
-			self.export_blocks)
 
 		self.connect(self.ui.movement,
 			QtCore.SIGNAL("highlighted(int)"),
