@@ -1,5 +1,6 @@
 from PyQt4 import QtGui
 from Browser import Browser
+from ProgressionDialog import ProgressionDialog
 
 DEFAULT = 1
 OWN = 2
@@ -17,6 +18,15 @@ class ProgressionBrowser(Browser):
 		self.ui.authors.addItem("All progressions")
 		Browser.setup(self)
 
+	def new_item(self):
+		i = QtGui.QListWidgetItem()
+		id = ProgressionDialog(i)
+		id.show()
+		id.exec_()
+		if i.text() != "":
+			self.listwidget.addItem(i)
+			self.reject()
+
 
 		
 	
